/************************************************************************
 ************************************************************************
                                                                         
 Module : bitmap.c                                                               
                                                                         
                                                                         
                                                                         
 Changes :                                                               

 8/9  S.A Include debug functions and assert. 
 9/9  S.A Add some comments.
 15/9 S.A New call to 'colourArray' from array<..> to colourArray<..>
 23/9 S.A Change some calls because new structure in colourArray. 'The size'.
 
************************************************************************* 
*************************************************************************/

/* included lib */
#include "bitmap.h"
#include "colour.h"
#include "scanLine.h"
#include "debug.h"
#include "errorHandler.h"
#include <assert.h>
#include <stdlib.h>
#include <stdio.h>


static const short MIN_WIDTH  = 1;
static const short MIN_HEIGHT = 1;
static const short MIN_POS    = 0;


/*-------------------------------------------------------------
 Function : bitmapMake                                                    
                                                               
 Purpus : Instansiate a bitmap structure. If mem empty
          then raise exception.                                                      
                                                               
                                                               
--------------------------------------------------------------*/
Bitmap bitmapMake(int width, int height)
{
  /* local variable */
  Bitmap bitmap;
	int i; 

  /* Just for debug */
  assert(width >= MIN_WIDTH && height >= MIN_HEIGHT);
  debugBitmap("bitmapMake");
	
	bitmap.rows = malloc(sizeof(ScanLine) * height);
	if (bitmap.rows == NULL)
    	errorHandler("bitmapMake", "out of memory for bitmap");

for  (i = 0 ; i < height ; i++)
	{
		bitmap.rows[i] = scanLineMake(width);
	}		
		

  bitmap.width       = width;
  bitmap.height      = height;

  debugExit();
  return bitmap;
  
}


/*-------------------------------------------------------------
 Function : bitmapDelete                                                   
                                                               
 Purpus : Free the memory that is allocated by the colourArray
                                                               
                                                               
--------------------------------------------------------------*/
void bitmapDelete(Bitmap bitmap)
{
  /* local variable */
	int i;


  /* just for debug */
  debugBitmap("bitmapDelete");

	for (i = 0 ; i < bitmap.height ; i++)
		scanLineDelete(bitmap.rows[i]);
  
  free(bitmap.rows);

  debugExit();
  return ;
}



/*-------------------------------------------------------------
 Function : bitmapSetColour                                                   
                                                               
 Purpus : Given a bitmap and a colour. The colour is inserted
          at position xPos,yPos in bitmap.
	  Syntax: of 3x3 bitArray:

	Y-Axis:  |   1     |    2    |    3    |
	         |---------|---------|---------|
	         |         |         |         |
	         |---------|---------|---------|
    X-Axis   |1   2  3 | 1   2  3| 1   2  3|
                                                               
--------------------------------------------------------------*/
Bitmap bitmapSetColour(Bitmap bitmap, Colour colour, int xPos, int yPos)
{
  /* local variable */

  
  /* Just for debug */
  assert(xPos >= MIN_POS && 
	 yPos >= MIN_POS &&
	 xPos < bitmap.width &&
	 yPos < bitmap.height); 

  debugBitmap("bitmapSetColour");
  
   
  scanLineInsertElement(bitmap.rows[yPos] , colour , xPos);
 

  debugExit();
  return bitmap;
    
}


/*-------------------------------------------------------------
 Function : bitmapGetColour                                                   
                                                               
 Purpus : Return the colour in xPos,yPos.
          Test input X,Y value to see if they are in
	  valid range. MIN_POS <= (X,Y) <= MAX_POS
                                                               
--------------------------------------------------------------*/
Colour bitmapGetColour(Bitmap bitmap, int xPos, int yPos)
{
  /* local variable */
  Colour      colour;


  
  /* Just for debug */
  debugBitmap("bitmapGetColour");
  assert(xPos >= MIN_POS && 
	 yPos >= MIN_POS &&
	 xPos <= bitmap.width &&
	 yPos <= bitmap.height);    


  colour = scanLineGetElement(bitmap.rows[yPos] , xPos);

  debugExit();

  return colour;
}


/*-------------------------------------------------------------
 Function : bitmapWidth                                                   
                                                               
 Purpus :                                                      
                                                               
                                                               
--------------------------------------------------------------*/
int bitmapWidth(Bitmap bitmap)
{
  /* Just for debug */
  debugBitmap("bitmapWidht");
  debugExit();


  return bitmap.width;
}

/*-------------------------------------------------------------
 Function : bitmapHeight                                                    
                                                               
 Purpus :                                                      
                                                               
                                                               
--------------------------------------------------------------*/
int bitmapHeight(Bitmap bitmap)
{
  /* Just for debug */
  debugBitmap("bitmapHeight");
  debugExit();
  


  return bitmap.height;
}


/*-------------------------------------------------------------
 Function : bitmapSetRow                                                  
                                                               
 Purpus :                                                      
                                                               
                                                               
--------------------------------------------------------------*/
Bitmap bitmapSetRow(unsigned char * buffer , Bitmap bmp , int Line)
{
	int width;
	int height;
	int x;
	int r,g,b;
	Colour col;

  debugBitmap("bitmapHeight");

	
	width  = bitmapWidth(bmp) * 3;
	height = bitmapHeight(bmp);
	
	for (x = 0 ; x < width ; x++)
	{
		r = buffer[x];
		if (r<0)
			r = 255 + r;
		x++;
		g = buffer[x];
		if (g<0)
			g = 255 + g;
		x++;
		b = buffer[x];
		if (b<0)
			b = 255 + b;
			
		col = colourMake(r,g,b);		
		bitmapSetColour(bmp , col , (x+1)/3 - 1 , Line);
	}
	
  debugExit();

	return bmp;
}

Bitmap bitmapSetLine(Bitmap bmp , int line , ScanLine scanLine)
{
	debugBitmap("bitmapSetLine");
	
	scanLineDelete(bmp.rows[line]);
	
	bmp.rows[line] = scanLine;
	
	debugExit();
	return bmp;
}


ScanLine bitmapGetLine(Bitmap bmp , int line)
{
	ScanLine scanLine;
	
	debugBitmap("bitmapGetLine");
	
	scanLine = bmp.rows[line];
	
	debugExit();
	
	return scanLine;
}		


Bitmap bitmapResizeCol(Bitmap bitmap , int height , Bitmap smallbmp)
{
	int oldHeight , width , line , x;
	float pixHeight , scale , rest;
	float y , leftofpix ,currenty;
	Colour newcol;
	ColourAcc colacc;
		
	debugBitmap("bitmapWidht");

	oldHeight = bitmapHeight(bitmap);
	width     = bitmapWidth(bitmap);

	pixHeight = (float)oldHeight / (float)height;
	
	scale = 1.0 / pixHeight;
	
	for (x = 0 ; x < width ; x++)
	{
		y = 0;
		rest = 1;
		if (rest > pixHeight)
			rest = pixHeight;

		for (currenty = 0 ; currenty < height ; currenty++)
		{
			leftofpix = pixHeight;
	
			newcol = bitmapGetColour(bitmap , x , y);
			colacc = colourToColourAcc(colourScale(newcol , rest));
			leftofpix = leftofpix - rest;
			rest = 0;
	

			while (leftofpix > 1.0  && (int)(y + pixHeight - leftofpix) < oldHeight)
				{	
				
					newcol = bitmapGetColour(bitmap , x , (int)(y + pixHeight - leftofpix));
					colacc = colourUnion(colacc , newcol);
					leftofpix = leftofpix - 1.0;
				}
			if (leftofpix > 0.0 )
				{
					line = (int)(y + pixHeight - leftofpix);
					if (line >= oldHeight)
							line = oldHeight - 1;
					newcol = bitmapGetColour(bitmap , x , line);
					newcol = colourScale(newcol , leftofpix);
					colacc = colourUnion(colacc , newcol);
				}
				
			rest = 1.0 - leftofpix;
			if (rest > pixHeight)
				rest = pixHeight;

			newcol = colourAccScale(colacc , scale);
			bitmapSetColour(smallbmp , newcol , x , currenty);
			y = y + pixHeight;
		}
	}
	
	debugExit();

	return smallbmp;
}
	





Bitmap bitmapResize(Bitmap bmp , int width , int height)
{
	int oldWidth , oldHeight , line , column;
	float pixHeight , pixWidth , scale , rest;
	float x , y , i , leftofpix , currentx , currenty;
	Colour newcol;
	ColourAcc colacc;
	Bitmap tmpbmp , smallbmp;
	
  debugBitmap("bitmapWidht");

	oldWidth  = bitmapWidth(bmp);
	oldHeight = bitmapHeight(bmp);

	if (height == 0)
		height = oldHeight * (float)width / (float)oldWidth;
	if (width == 0)
		width = oldWidth * (float)height / (float)oldHeight;
	
	pixWidth  = (float)oldWidth / (float)width;
	pixHeight = (float)oldHeight / (float)height;
	
	printf("oldWidth: %d\noldHeight: %d\n",oldWidth,oldHeight);
	
	tmpbmp = bitmapMake(width , oldHeight);
	
	scale = 1.0 / pixWidth;
	
	for (y = 0 ; y < oldHeight ; y++)
	{
		currentx = 0;
		rest = 0;
		for (x = 0 ; x < oldWidth ; x = x + pixWidth)
		{
			leftofpix = pixWidth;
			currentx++;

		
			newcol = bitmapGetColour(bmp , x , y);
			colacc = colourToColourAcc(colourScale(newcol , rest));
			leftofpix = leftofpix - rest;
			rest = 0;
		

			while (leftofpix > 1.0)
				{	
					newcol = bitmapGetColour(bmp , (int)(x + pixWidth - leftofpix) , y);
					colacc = colourUnion(colacc , newcol);
					leftofpix = leftofpix - 1.0;
				}

			if (leftofpix > 0.0 )
				{
					i = leftofpix;
					column = (int)(x + pixWidth - leftofpix);
					if (column > oldWidth)
							column = oldWidth;
					newcol = bitmapGetColour(bmp , column , y);
					newcol = colourScale(newcol , i);
					colacc = colourUnion(colacc , newcol);
				}
				
			rest = 1.0 - leftofpix;
			if (rest > pixWidth)
				rest = pixWidth;

			newcol = colourAccScale(colacc , scale);
			bitmapSetColour(tmpbmp , newcol , currentx , y);
		}
	}
	bitmapDelete (bmp);

	smallbmp = bitmapMake(width , height);
	scale = 1.0 / pixHeight;
	
	for (x = 0 ; x < width ; x++)
	{
		currenty = 0;
		rest = 0;
		for (y = 0 ; y < oldHeight ; y = y + pixHeight)
		{
			leftofpix = pixHeight;
	
			newcol = bitmapGetColour(tmpbmp , x , y);
			colacc = colourToColourAcc(colourScale(newcol , rest));
			leftofpix = leftofpix - rest;
			rest = 0;
	

			while (leftofpix > 1.0)
				{	
					newcol = bitmapGetColour(tmpbmp , x , (int)(y + pixHeight - leftofpix));
					colacc = colourUnion(colacc , newcol);
					leftofpix = leftofpix - 1.0;
				}

			if (leftofpix > 0.0 )
				{
					line = (int)(y + pixHeight - leftofpix);
					if (line > oldHeight)
							line = oldHeight;
					i = leftofpix;
					newcol = bitmapGetColour(tmpbmp , x , line
					);
					newcol = colourScale(newcol , i);
					colacc = colourUnion(colacc , newcol);
				}
				
			rest = 1.0 - leftofpix;
			if (rest > pixWidth)
				rest = pixWidth;

			newcol = colourAccScale(colacc , scale);
			bitmapSetColour(smallbmp , newcol , x , currenty);
			currenty++;
		}
	}
	bitmapDelete (tmpbmp);
  debugExit();

  debugExit();
	return smallbmp;
}


Bitmap bitmapSetJsample(unsigned char * jsample , Bitmap bmp , int line)
{
	int width;
	int height;
	int x;
	int r,g,b;
	Colour col;
	
  debugBitmap("bitmapWidht");

	width  = bitmapWidth(bmp) * 3;
	height = bitmapHeight(bmp);
	
	for (x = 0 ; x < width ; x++)
	{
		r = jsample[x];
		if (r<0)
			r = 255 + r;
		x++;
		g = jsample[x];
		if (g<0)
			g = 255 + g;
		x++;
		b = jsample[x];
		if (b<0)
			b = 255 + b;
		col = colourMake(r,g,b);		
		bitmapSetColour(bmp , col , (x+1)/3 , line);
	}
	
  debugExit();
	return bmp;
}


unsigned char * bitmapGetJsample(Bitmap bmp ,int line)
{
	ScanLine scanLine;
	
	unsigned char * jsample;
	
	debugBitmap("bitmapWidth");

	scanLine = bitmapGetLine (bmp , line);

	
	jsample = scanLineToJsample(scanLine);
	
  debugExit();
	return jsample;
}
	



/*-------------------------------------------------------------
 Function : bitmapPrint                                                   
                                                               
 Purpus :                                                      
                                                               
--------------------------------------------------------------*/
void bitmapPrint(Bitmap bitmap)
{
  /* local variable */
  Colour      colour;
  int         width, height, x, y;


  
  width       = bitmap.width;
  height      = bitmap.height;
    

  printf("width: %d\nheight: %d\n", width , height);
  for(x = 0, y = 0; x < width && y < height; x++,y++)
    {
      colour = bitmapGetColour(bitmap, x, y);
      colourPrint(colour);
    }

  return ;
}
  

