
/************************************************************************
 ************************************************************************
                                                                         
 Module : colour.c                                                               
                                                                         
                                                                         
                                                                         
 Changes :                                                               
         
 8/9  S.A Include debug functions and assert. 
 9/9  S.A Add some comments.
 13/9 S.A Remove function 'colourDelete'. No mix between heap and stack 
          structures

 19/9 S.A Change ADT-syntax.
 19/9 S.A Add function that translate from float to colorValue
 21/9 S.A New type of colourValue from int to unsign char
 26/9 S.A Added colourDefaultValue.
*************************************************************************
*************************************************************************/

/* included lib */
#include "colour.h"
#include "debug.h"
#include "scalar.h"
#include <stdio.h>
#include <assert.h>
#include <math.h>

/* Converter */
#define toColourValue(value) (((ColourValue)(value)))
#define toInt(value) (((int)(value)))
#define toFloat(value) (((Scalar)(value)))


static const ColourValue MIN_COLOUR_VALUE     = 0;
static const ColourValue MAX_COLOUR_VALUE     = 255;




/*-------------------------------------------------------------
 Function : colourGetRedValue                                                   
                                                               
 Purpus : 
                                                               
                                                               
--------------------------------------------------------------*/
ColourValue colourGetRedValue(Colour colour)
{
  ColourValue col;
 
  /* Just for debug */
  debugColour("colourGetRedValue");
 

  col = (ColourValue)((unsigned char *)&colour)[0];
  
  debugExit();

  return col;
}



/*-------------------------------------------------------------
 Function : colourGetGreenValue                                                   
                                                               
 Purpus : 
                                                               
                                                               
--------------------------------------------------------------*/
ColourValue colourGetGreenValue(Colour colour)
{
  ColourValue col;
  /* Just for debug */
  debugColour("colourGetGreenValue");
  
  col = (ColourValue)((unsigned char *)&colour)[1];
  
  debugExit();

  return col;
}


 


/*-------------------------------------------------------------
 Function : colourGetBlueValue                                                   
                                                               
 Purpus : 
                                                               
                                                               
--------------------------------------------------------------*/
ColourValue colourGetBlueValue(Colour colour)
{
  ColourValue col;
  
  /* Just for debug */
  debugColour("colourGetBlueValue");

  col = (ColourValue)((unsigned char *)&colour)[2];

  debugExit();

  return col;
}


/*-------------------------------------------------------------
 Function : colourIntToColourValue                                                   
                                                               
 Purpus : Convert a integer to a colourValue                                                     
                                                               
                                                               
--------------------------------------------------------------*/
ColourValue colourIntToColourValue(int value)
{
  /* Just for debug */
  assert(value >= MIN_COLOUR_VALUE &&
	 value <= MAX_COLOUR_VALUE);

  debugColour("intToColourValue");
  debugExit();

  return toColourValue(value);
}


/*-------------------------------------------------------------
 Function : colourFloatToColourValue                                                               
 Purpus :                                                       
                                                               
                                                               
--------------------------------------------------------------*/
ColourValue colourFloatToColourValue(float value)
{
  /* local variable */
  ColourValue colourValue;
    
  /* Just for debug */
  debugColour("colourFloatToColourValue");
  debugExit();

  colourValue = toColourValue(value * MAX_COLOUR_VALUE);
  if(colourValue > MAX_COLOUR_VALUE)
    colourValue = MAX_COLOUR_VALUE;
  
  return colourValue;
}


/*-------------------------------------------------------------
 Function : colourValueToInt                                                   
                                                               
 Purpus : Convert a colourValue to integer                                                     
                                                               
                                                               
--------------------------------------------------------------*/
static int colourValueToInt(ColourValue value)
{
  /* Just for debug */
  debugColour("colourValueToInt");
  debugExit();
  
  return toInt(value);
}




/*-------------------------------------------------------------
 Function : colourSetValue                                                   
                                                               
 Purpus : Must test if the value is in correct range.
          MIN_COLOUR_VALUE <= (r,g,b) <= MAX_COLOUR_VALUE
                                                               
                                                               
--------------------------------------------------------------*/
Colour colourSetValue(Colour colour, 
		      ColourValue red, 
		      ColourValue green, 
		      ColourValue blue)
{

	unsigned char col[4];

  /* Just for debug */
  assert(red   >= MIN_COLOUR_VALUE &&
		 green >= MIN_COLOUR_VALUE &&
		 blue  >= MIN_COLOUR_VALUE &&
		 red   <= MAX_COLOUR_VALUE &&
		 green <= MAX_COLOUR_VALUE &&
		 blue  <= MAX_COLOUR_VALUE);
  
  debugColour("colourSetValue");

	col[0] = red;
	col[1] = green;
	col[2] = blue;

  colour = (Colour)col;
  
  
  debugExit();
  return colour;
}



/*-------------------------------------------------------------
 Function : colourMake                                                   
                                                               
 Purpus : Must test if the value is in correct range.
          MIN_COLOUR_VALUE <= (r,g,b) <= MAX_COLOUR_VALUE
                                                     
                                                               
                                                               
--------------------------------------------------------------*/
Colour colourMake(ColourValue red, ColourValue green, ColourValue blue)
{ 
  /* local variable */
  Colour        *col1;
  Colour        col2;
  unsigned char col[4];

	/* just for debug */  
  debugColour("colourMake");  

	col[0] = red;
	col[1] = green;
	col[2] = blue;

  col1 = (Colour*)col;

  col2 = (Colour)*col1;

  debugExit();
  return col2;
}


/*-------------------------------------------------------------
 Function : colourAccMake                                                   
                                                               
 Purpus : Must test if the value is in correct range.
          MIN_COLOUR_VALUE <= (r,g,b) <= MAX_COLOUR_VALUE
                                                     
                                                               
                                                               
--------------------------------------------------------------*/
ColourAcc colourAccMake(ColourValue red, ColourValue green, ColourValue blue)
{ 
  /* local variable */
	ColourAcc colAcc
  
	/* just for debug */  
  debugColour("colourMake");  

	colAcc.red   = red;
	colAcc.green = green;
	colAcc.blue  = blue;

  debugExit();

  return colAcc;
}



/*-------------------------------------------------------------
 Function : colourUnion                                                    
                                                               
 Purpus : Make union betweenn two colours and return the new
          colour that appears after the union.
	  Do not need to test if colour newValue < MIN_COLOUR_VALUE
	  because no colourValue can be negative.
                                                               
                                                               
--------------------------------------------------------------*/
ColourAcc colourUnion(ColourAcc colourOne, Colour colourTwo)
{
  /* local variable */
  ColourValue    newRed;
  ColourValue    newGreen;
  ColourValue    newBlue;
  ColourAcc      newColour;



  /* Just for debug */
  debugColour("colourUnion");
  

  newRed = 
    colourOne.red   + colourGetRedValue(colourTwo); 
  
  newGreen = 
    colourOne.green + colourGetGreenValue(colourTwo);
  
  newBlue = 
    colourOne.blue  + colourGetBlueValue(colourTwo);
  
   
  newColour = colourAccMake(newRed, newGreen, newBlue);

  debugExit();
  return newColour;
}



/*-------------------------------------------------------------
 Function : colourScale                                                 
                                                               
 Purpus : returns the the same colour modified with a
          percentage scale at the colour
                                                               
--------------------------------------------------------------*/
Colour colourScale(Colour colour ,float scale)
{
  /* local variable */
  Colour         newColour;
  float          control;
  ColourValue    newRed;
  ColourValue    newGreen;
  ColourValue    newBlue;


  
  /* Just for debug */
  debugColour("colourScale");
	


  control   = scale;
	if (control < 0.0)
		{
			printf("Negative scale value\n");
			exit(0);
		}
	if (control >= 1.0)
		{
			return colour;
			/* Could look like this depending on what's correct
  			newRed    = toColourValue(colourGetRedValue(colour)   * control);
			if (newRed > MAX_COLOUR_VALUE)
				newRed = MAX_COLOUR_VALUE;

			newGreen  = toColourValue(colourGetGreenValue(colour) * control);
			if (newGreen > MAX_COLOUR_VALUE)
				newGreen = MAX_COLOUR_VALUE;

			newBlue   = toColourValue(colourGetBlueValue(colour)  * control);
			if (newBlue > MAX_COLOUR_VALUE)
				newBlue = MAX_COLOUR_VALUE;
			*/
		}
	else
		{
			newRed    = toColourValue(colourGetRedValue(colour)   * control);
			newGreen  = toColourValue(colourGetGreenValue(colour) * control);
			newBlue   = toColourValue(colourGetBlueValue(colour)  * control);
		}
		
		
  newColour = colourMake(newRed, newGreen, newBlue);
 


  
  debugExit();
  return newColour;
}
/*-------------------------------------------------------------
 Function : colourAccScale                                                 
                                                               
 Purpus : returns the the same colour modified with a
          percentage scale at the colour
                                                               
--------------------------------------------------------------*/
Colour colourAccScale(ColourAcc colour ,float scale)
{
  /* local variable */
  Colour         newColour;
  float          control;
  ColourValue    newRed   = 0;
  ColourValue    newGreen = 0;
  ColourValue    newBlue  = 0;


  
  /* Just for debug */
  debugColour("colourScale");
	


  control   = scale;
	if (control < 0.0)
		{
			printf("Negative scale value\n");
			exit(0);
		}
	if (control > 1.0)
		{
  			newRed    = toColourValue(colour.red   * control);
			if (newRed > MAX_COLOUR_VALUE)
				newRed = MAX_COLOUR_VALUE;

			newGreen  = toColourValue(colour.green * control);
			if (newGreen > MAX_COLOUR_VALUE)
				newGreen = MAX_COLOUR_VALUE;

			newBlue   = toColourValue(colour.blue  * control);
			if (newBlue > MAX_COLOUR_VALUE)
				newBlue = MAX_COLOUR_VALUE;
		}
	else
		{
			newRed    = toColourValue(colour.red   * control);
			newGreen  = toColourValue(colour.green * control);
			newBlue   = toColourValue(colour.blue  * control);
		}
		
		
  newColour = colourMake(newRed, newGreen, newBlue);
 


  
  debugExit();
  return newColour;
}


ColourAcc colourToColourAcc(Colour colour)
{
	ColourAcc colacc;
	
	colacc = colourAccMake(	colourGetRedValue(colour),
							colourGetGreenValue(colour),
							colourGetBlueValue(colour));
					

	return colacc;
	
}



/* Test function */

/*-------------------------------------------------------------
 Function : colourPrint                                                    
                                                               
 Purpus : debug                                                    
                                                               
                                                               
--------------------------------------------------------------*/
void colourPrint(Colour colour)
{
  /* local variable */
  int red;
  int green;
  int blue;


  red   = colourValueToInt(colourGetRedValue(colour));
  green = colourValueToInt(colourGetGreenValue(colour));
  blue  = colourValueToInt(colourGetBlueValue(colour));
    
  printf("%d %d %d\n", red, green, blue); 
  return ;
}





