/************************************************************************
 ************************************************************************
                                                                         
 Module : colourArray.c                                                                
                                                                         
                                                                         
                                                                         
 Changes :                                                               
                  
 8/9  S.A Include debug functions and assert.
 9/9  S.A Add some comments.
 15/9 S.A Add function colourArrayLength.
 19/6 S.A Change ADT-syntax.
 23/6 S.A Change struct ColourArray. Added arraySize.
 
************************************************************************* 
*************************************************************************/

/* included lib */
#include "colour.h"
#include "colourArray.h"
#include "errorHandler.h"
#include "debug.h"
#include <assert.h>
#include <stdlib.h>


static const int MIN_ARR_SIZE = 1;
static const long MAX_ARR_SIZE  = 40000000;

/*-------------------------------------------------------------
 Function : colourArrayMake
 
 Purpus : The size must be in range
          MIN_ARR_SIZE <= size <= MAX_ARR_SIZE

 
                                                               
--------------------------------------------------------------*/
ColourArray colourArrayMake(int size)
{ 
  /* local variable */
  ColourArray colourArray;
  Array       array;

  
  /* Just for debug */
  assert(size >= MIN_ARR_SIZE && size <= MAX_ARR_SIZE);
  debugColourArray("colourArrayMake");
  
  array  = malloc(sizeof(Colour) * size);    
  
  if (array == NULL)
    errorHandler("colourArrayMake", "out of memory for colourArray");
  
  colourArray.arraySize = size;
  colourArray.array     = array;

  
  debugExit();
  return colourArray;
}



/*-------------------------------------------------------------
 Function : colourArrayDelete                                                    
                                                               
 Purpus : Free allocate memory that was occupied by the colourArray


 
                                                               
--------------------------------------------------------------*/
void colourArrayDelete(ColourArray colourArray)
{
  /* Just for debug */
  debugColourArray("colourArrayFree");
  
  free(colourArray.array);

  debugExit();
  return ;
}
 



/*-------------------------------------------------------------
 Function : colourArrayGetElement                                                    
                                                               
 Purpus : get the nth element in the array. 
          Index from 0 -> N-1. Upp to caller to 
          ensure that index is in array.                                                               
          Must see that index is in valid range
	  MIN_ARR_SIZE <= index <= MAX_ARR_SIZE or
	  arraySize <= index.
--------------------------------------------------------------*/
Colour colourArrayGetElement(ColourArray colourArray, int index)
{ 
  /* local variable */
  Colour element;
    
  /* Just for debug */
  assert(index >= MIN_ARR_SIZE - 1 &&
	 index <= MAX_ARR_SIZE - 1 &&
	 index <= colourArray.arraySize);
  
  debugColourArray("colourArrayGetElement");
	 
  element = colourArray.array[index];

  
  debugExit();
  return element;
}                                                                         


/*-------------------------------------------------------------
 Function : colourArrayInsertElement                                                   
                                                               
 Purpus : Insert a colour element at index position.
          It is up to caller to insure that index is correct position.
                                                               
--------------------------------------------------------------*/
ColourArray colourArrayInsertElement(ColourArray colourArray,
				     Colour element,
				     int index)
{
  /* Just for debug */
  assert(index >= MIN_ARR_SIZE - 1      &&
	 index <= MAX_ARR_SIZE - 1      &&
         index <= colourArray.arraySize);

  
  debugColourArray("colourArrayInsertElement");


  colourArray.array[index] = element;
 
  debugExit();
  return colourArray;
}


/*-------------------------------------------------------------
 Function : colourArrayLength
 
 Purpus :                                                       
                                                               
                                                               
--------------------------------------------------------------*/
int colourArrayLength(ColourArray colourArray)
{
  /* local variable */
  int size;
    
  /* Just for debug */
  debugColourArray("colourArrauLength");
  

  size = colourArray.arraySize;

  debugExit();
  return size;
}



ColourArray colourArrayFromJsample(unsigned char * jsample , int length)
{
	int x;
	int r,g,b;
	Colour col;
	ColourArray colArr;
	
	colArr = colourArrayMake(length / 3);
	
	for (x = 0 ; x < length ; x++)
	{
		r = jsample[x];
		if (r<0)
			r = 255 + r;
		x++;
		g = jsample[x];
		if (g<0)
			g = 255 + g;
		x++;
		b = jsample[x];
		if (b<0)
			b = 255 + b;
		col = colourMake(r,g,b);	
			
		colourArrayInsertElement(colArr , col , (x+1)/3);
	}
	
	return colArr;
}


unsigned char * colourArrayToJsample(ColourArray colArr)
{
	unsigned char * jsample;
	int jsampleLength , length;
	int x;
	Colour col;
	
	length = colourArrayLength(colArr);
	jsampleLength = length * 3;
	
	jsample = (unsigned char *)malloc(sizeof(unsigned char) * jsampleLength);    
  
  	if (jsample == NULL)
    	errorHandler("colourArrayMake", "out of memory for jsample");

	for (x = 0 ; x < length ; x++)
	{
		col = colourArrayGetElement(colArr , x);
		jsample[x*3]     = (unsigned char)colourGetRedValue(col);

		jsample[x*3 + 1] = (unsigned char)colourGetGreenValue(col);

		jsample[x*3 + 2] = (unsigned char)colourGetBlueValue(col);
	}
	

	return jsample;

}






