/************************************************************************
 ************************************************************************
                                                                         
 Module : scanLine.c                                                                
                                                                         
                                                                         
                                                                         
 Changes :                                                               
                  
 8/9  S.A Include debug functions and assert.
 9/9  S.A Add some comments.
 15/9 S.A Add function scanLineLength.
 19/6 S.A Change ADT-syntax.
 23/6 S.A Change struct ScanLine. Added size.
 
************************************************************************* 
*************************************************************************/

/* included lib */
#include "colour.h"
#include "scanLine.h"
#include "errorHandler.h"
#include "debug.h"
#include <assert.h>
#include <stdlib.h>
#include <stdio.h>


static const int MIN_ARR_SIZE = 1;
static const long MAX_ARR_SIZE  = 40000000;

/*-------------------------------------------------------------
 Function : scanLineMake
 
 Purpus : The size must be in range
          MIN_ARR_SIZE <= size <= MAX_ARR_SIZE

 
                                                               
--------------------------------------------------------------*/
ScanLine scanLineMake(int size)
{ 
  /* local variable */
  ScanLine scanLine;
  Array       array;

  
  /* Just for debug */
  assert(size >= MIN_ARR_SIZE && size <= MAX_ARR_SIZE);
  debugScanLine("scanLineMake");
  
  array  = malloc(sizeof(Colour) * size);    
  
  if (array == NULL)
    errorHandler("scanLineMake", "out of memory for scanLine");
  
  scanLine.size = size;
  scanLine.array     = array;

  
  debugExit();
  return scanLine;
}



/*-------------------------------------------------------------
 Function : scanLineDelete                                                    
                                                               
 Purpus : Free allocate memory that was occupied by the scanLine


 
                                                               
--------------------------------------------------------------*/
void scanLineDelete(ScanLine scanLine)
{
  /* Just for debug */
  debugScanLine("scanLineFree");
  
  free(scanLine.array);

  debugExit();
  return ;
}
 



/*-------------------------------------------------------------
 Function : scanLineGetElement                                                    
                                                               
 Purpus : get the nth element in the array. 
          Index from 0 -> N-1. Upp to caller to 
          ensure that index is in array.                                                               
          Must see that index is in valid range
	  MIN_ARR_SIZE <= index <= MAX_ARR_SIZE or
	  size <= index.
--------------------------------------------------------------*/
Colour scanLineGetElement(ScanLine scanLine, int index)
{ 
  /* local variable */
  Colour element;
    
  /* Just for debug */
  assert(index >= MIN_ARR_SIZE - 1 &&
	 index < scanLine.size);
  
  debugScanLine("scanLineGetElement");
	 
  element = scanLine.array[index];

  
  debugExit();
  return element;
}                                                                         


/*-------------------------------------------------------------
 Function : scanLineInsertElement                                                   
                                                               
 Purpus : Insert a colour element at index position.
          It is up to caller to insure that index is correct position.
                                                               
--------------------------------------------------------------*/
ScanLine scanLineInsertElement(ScanLine scanLine,
				     Colour element,
				     int index)
{
  /* Just for debug */
  assert(index >= MIN_ARR_SIZE - 1      &&
         index < scanLine.size);

  
  debugScanLine("scanLineInsertElement");


  scanLine.array[index] = element;
 
  debugExit();
  return scanLine;
}


/*-------------------------------------------------------------
 Function : scanLineLength
 
 Purpus :                                                       
                                                               
                                                               
--------------------------------------------------------------*/
int scanLineLength(ScanLine scanLine)
{
  /* local variable */
  int size;
    
  /* Just for debug */
  debugScanLine("scanLineLength");
  

  size = scanLine.size;

  debugExit();
  return size;
}




/*-------------------------------------------------------------
 Function : scanLineFromJsample
 
 Purpus :                                                       
                                                               
                                                               
--------------------------------------------------------------*/
ScanLine scanLineFromJsample(unsigned char * jsample , int length ,
								ScanLine sLine)
{
	int x;
	int r,g,b;
	Colour col;
	
	
	for (x = 0 ; x < length ; x++)
	{
		r = jsample[x];
		if (r<0)
			r = 255 + r;
		x++;
		g = jsample[x];
		if (g<0)
			g = 255 + g;
		x++;
		b = jsample[x];
		if (b<0)
			b = 255 + b;
		col = colourMake(r,g,b);
		scanLineInsertElement(sLine , col , (x+1)/3 - 1);
	}
	
	return sLine;
}



/*-------------------------------------------------------------
 Function : scanLineToJsample
 
 Purpus :                                                       
                                                               
                                                               
--------------------------------------------------------------*/
unsigned char * scanLineToJsample(ScanLine colArr)
{
	unsigned char * jsample;
	int jsampleLength , length;
	int x;
	Colour col;
	
	length = scanLineLength(colArr);
	jsampleLength = length * 3;
	
	jsample = (unsigned char *)malloc(sizeof(unsigned char) * jsampleLength);    
  
  	if (jsample == NULL)
    	errorHandler("scanLineMake", "out of memory for jsample");

	for (x = 0 ; x < length ; x++)
	{
		col = scanLineGetElement(colArr , x);
		jsample[x*3]     = (unsigned char)colourGetRedValue(col);

		jsample[x*3 + 1] = (unsigned char)colourGetGreenValue(col);

		jsample[x*3 + 2] = (unsigned char)colourGetBlueValue(col);
	}
	

	return jsample;

}


ScanLine scanLineResize(ScanLine scanLine , int width , ScanLine newLine)
{
  int oldSize , column;
  float pixWidth , scale , rest;
  float x , leftofpix , currentx;
  Colour		newcol;
  ColourAcc	colacc;
  
  debugScanLine("scanLineResize");
  
  oldSize  = scanLine.size;
  
  pixWidth  = (float)oldSize / (float)width;
  scale = 1.0 / pixWidth;
  
  currentx = 0;
  rest = 1;
  if (rest > pixWidth)
    rest = pixWidth;
  
  for (x = 0 ; x < oldSize ; x = x + pixWidth)
    {
      leftofpix = pixWidth;
      
      
      newcol = scanLineGetElement(scanLine , (int)x);
      colacc = colourToColourAcc(colourScale(newcol , rest));
      leftofpix = leftofpix - rest;
      rest = 0;
		

      while (leftofpix > 1.0)
	{	
	  newcol = scanLineGetElement(scanLine , (int)(x + pixWidth - leftofpix));
	  colacc = colourUnion(colacc , newcol);
	  leftofpix = leftofpix - 1.0;
	}
      
      if (leftofpix > 0.0 )
	{
	  column = (int)(x + pixWidth - leftofpix);
	  if (column >= oldSize)
	    column = oldSize - 1;
	  newcol = scanLineGetElement(scanLine , column);
	  newcol = colourScale(newcol , leftofpix);
	  colacc = colourUnion(colacc , newcol);
	}
      
      rest = 1.0 - leftofpix;
      if (rest > pixWidth)
	rest = pixWidth;
      
      newcol = colourAccScale(colacc , scale);
      scanLineInsertElement(newLine , newcol , currentx);
      currentx++;
    }

return newLine;
}
	








